/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007, 2008. All Rights Reserved. 
 * 
 * Note to U.S. Government Users Restricted Rights:  Use, 
 * duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.require("dojo.date.locale");

/* Constants for parameter types */
var TYPE_ANY = 0;
var TYPE_STRING = 1;
var TYPE_FLOAT = 2;
var TYPE_DECIMAL = 3;
var TYPE_DATE_TIME = 4;
var TYPE_BOOLEAN = 5;
var TYPE_INTEGER = 6;
var TYPE_DATE = 7;
var TYPE_TIME = 8;

var HIERARCHICAL = 1 << 6;
var TEAM_AREA_NAME = 1 << 8;
var CONTRIBUTOR_NAME = 1 << 9;
var CONTRIBUTOR_USERID = 1 << 10;
var ITERATION_NAME = 1 << 11;
var TEAM_AREA_ITEMID = 1 << 15;
var ITERATION_ITEMID = 1 << 17;
var MY_TEAM_AREAS = "{My Team Areas}";
var CURRENT_ITERATION = "{Current Iteration}";
var CURRENT_PROJECT_AREA = "{Current Project Area}";
var CURRENT_CONTRIBUTOR = "{Current Contributor}";

/* Whether or not we are in edit mode */
var inEditMode = false;

/* Whether or not the report is dirty */
var isDirty = false;

/* Support for the Save As... floating dialog */
var dragOK = false;
var savedTarget = null;
var dragXoffset = 0;
var dragYoffset = 0;

/* A list of active Dojo widgets. These are destroyed when the section is collapsed */
var activeDojoWidgets = [];

/* Initialization methods */

// Hook events on page load.
function load() {
	var outerDiv = document.getElementById("parameterOuterDiv");
	if (isIe()) {
		outerDiv.attachEvent("onclick", handleEdit);
	} else {
		outerDiv.addEventListener("click", handleEdit, false);
	}
	var twistyLink = document.getElementById("twistyLink");
	if (isIe()) {
		twistyLink.attachEvent("onclick", handleTwisty);
	} else {
		twistyLink.addEventListener("click", handleTwisty, false);
	}
	var floatingPane = document.getElementById("floatingPane");
	if (isIe()) {
		floatingPane.attachEvent("onmousedown", dragHandler);
		floatingPane.attachEvent("onmouseup", endDragHandler);
		document.attachEvent("onmousemove", moveHandler);
	} else {
		floatingPane.addEventListener("mousedown", dragHandler, false);
		floatingPane.addEventListener("mouseup", endDragHandler, false);
		document.addEventListener("mousemove", moveHandler, false);
	}
	var exportFloatingPane = document.getElementById("exportFloatingPane");
	if (isIe()) {
		exportFloatingPane.attachEvent("onmousedown", exportDragHandler);
		exportFloatingPane.attachEvent("onmouseup", endDragHandler);
	} else {
		exportFloatingPane.addEventListener("mousedown", exportDragHandler, false);
		exportFloatingPane.addEventListener("mouseup", endDragHandler, false);
	}
}

// For each parameter, fetch the appropriate values from the rest service to
// store in the variable and to populate any combos or lists, and select 
// any pre-supplied values if appropriate.
function initializeParameters() {
	// For each top-level parameter in the report
    for (var i = 0; i < parameterList.length; i++) {
        var parameter = parameterList[i];
        if (parameter.isGroup == true) {
            initializeParameterGroup(parameter);
        } else if (isSpecialParameter(parameter.name)) {
        	initializeSpecialParameter(parameter.name);
        } else {
            initializeNonCascadingParameter(parameter);
        }
    }
}

// Initialize the parameter group.
// Since we never open in edit mode, this simply means to compute
// labels based on the initial parameter values and then fire
// off a request for the values.
function initializeParameterGroup(parameter) {
    if (!parameter.children || parameter.children.length == 0) return;

	// For each child parameter, compute a label based on the initial values.
	for (var i = 0; i < parameter.children.length; i++) {
		updateLabel(parameter.children[i]);
	}
	
    // Now make one call to the REST service for the entire group.
   	var oldReportMarker = reportMarker;
    var obj = {
        mySuccessFunction: function(args) {
        	// Check if we're still displaying this report.
        	if (!dojo.isDescendant(oldReportMarker, document.body)) {
        		return;
        	}
        	computeFlagsForGroup(args);
        	// Store the result in the appropriate variable.
        	var index = getParameterIndex(parameter.name);
        	parameterPotentialValues[index] = args.parameters[0];
        	parameterPotentialLabels[index] = args.parameters[0];

        	// If we are in edit mode, update the widgets with the new values.
        	if (inEditMode) {
			    var group = findGroup(parameter.name);
			    for (var i = 0; i < parameter.children.length; i++) {
			    	var child = parameter.children[i];
				    var element = document.getElementById(child.name + "_checkboxList_div");
				    if (element != null) {
					    updateCheckboxListContents(child.name, getParameterValues(child), getParameterLabels(child), element);
					} else {
						element = document.getElementById(child.name + "_select_div");
						if (element != null) {
							handleEditSelectSingle(child.name, getParameterValues(child), getParameterLabels(child), element);
						}
					}
				}
        	} else {
        		for (var i = 0; i < parameter.children.length; i++) {
	        		updateLabel(parameter.children[i]);
	        	}
        	}
        },
        myErrorFunction: function(errorType, errorMessage) {
        	alert("Error: " + errorType.message);
        }
    };
    callRestService("getParameterGroupValues", 
        {reportUUID: reportUUID, groupName: parameter.name, showArchived: showArchived}, obj, 
        "mySuccessFunction", "myErrorFunction");
}

// Return an appropriate label for insertion into the selected
// indicator node, of the form "x of y selected".
// If we don't know what "y" is, just say "x selected".
function getSelectedIndicatorLabel(parameterName) {
	var parameter = allParameterList[getAllParameterIndex(parameterName)];
	var selectedValues = getCurrentOrDefaultLabels(parameter);
	if (selectedValues == "''" || selectedValues == "") {
		selectedValues = null;
	} 
	if ((selectedValues != null) && !(selectedValues instanceof Array)) {
		selectedValues = [selectedValues];
	}
	var allValues = getParameterValues(parameter);
	if (allValues != null) {
		if (!(allValues instanceof Array)) {
			allValues = [allValues];
		}
		allValues = removeDuplicates(allValues);
	}
	if (allValues == null) {
		return "" + ((selectedValues == null) ? 0 : selectedValues.length) + " selected";
	} else {
		return "" + ((selectedValues == null) ? 0 : selectedValues.length) + " of " + allValues.length + " selected";
	}
}

// Install an appropriate label into the (collapsed) label widget
// for the given parameter. In the case that we have available
// labels that might be different from the values, use them instead.
function updateLabel(parameter) {
	// No label initialization to do for special parameters.
	if (!isSpecialParameter(parameter.name)) {
		var span = document.getElementById(parameter.name + "_select_div");
		if (span != null) {
			if (span.childNodes[0] != null) {
				span.removeChild(span.childNodes[0]);
			}
			span.appendChild(document.createTextNode(getLabelFor(getCurrentOrDefaultLabels(parameter))));
		} else {
			span = document.getElementById(parameter.name + "_checkboxList_div");
			if (span != null) {
				if (span.childNodes[0] != null) {
					span.removeChild(span.childNodes[0]);
				}
				span.appendChild(document.createTextNode(getLabelFor(getCurrentOrDefaultLabels(parameter))));
			} else {
				var values = getCurrentValues(parameter);
				if (values == null) {
					values = getDefaultValues(parameter);
				}
				span = document.getElementById(parameter.name + "_checkbox_div");
				if (span != null) {
					var label = null;
					if (values == null) {
						label = "(No selection)";
					} else {
						if (values == "true") {
							label = "Yes";
						} else {
							label = "No";
						}
					}
					if (span.childNodes[0] != null) {
						span.removeChild(span.childNodes[0]);
					}
					span.appendChild(document.createTextNode(label));
				} else {
					span = document.getElementById(parameter.name + "_text_div");
					if (span != null) {
						if (span.childNodes[0] != null) {
							span.removeChild(span.childNodes[0]);
						}
						if (parameter.concealed) {
							span.appendChild(document.createTextNode("********"));
						} else {
							span.appendChild(document.createTextNode(getLabelFor(values)));
						}
					} else {
						span = document.getElementById(parameter.name + "_date_div");
						if (span != null) {
							if (span.childNodes[0] != null) {
								span.removeChild(span.childNodes[0]);
							}
							span.appendChild(document.createTextNode(getLabelFor(values)));
						} else {
							alert("Unknown widget type for " + parameter.name);
						}
					}
				}
			}
		}
	}
}

// Initialize the parameter which is not part of a cascading group.
// Since we never open in edit mode, this simply means to compute
// a label based on the initial parameter values.
// Then call the REST service to queue up loading of the values
// since we may need them later.
function initializeNonCascadingParameter(parameter) {
	// Update the label
	updateLabel(parameter);
	
	// Now queue up the loading of the values
   	var oldReportMarker = reportMarker;
    var obj = {
        mySuccessFunction: function(args) {
        	// Check if we're still displaying this report.
        	if (!dojo.isDescendant(oldReportMarker, document.body)) {
        		return;
        	}

        	computeFlagsForParameter(args);
        	// Store the result in the appropriate variable.
        	var index = getParameterIndex(parameter.name);
        	parameterPotentialValues[index] = args.values;
        	parameterPotentialLabels[index] = args.labels;
        	
        	if (inEditMode) {
	        	// If we are in edit mode when the values come back, update the widgets.
			    var element = document.getElementById(parameter.name + "_checkboxList_div");
			    if (element != null) {
				    updateCheckboxListContents(parameter.name, getParameterValues(parameter), getParameterLabels(parameter), element);
				} else {
					element = document.getElementById(parameter.name + "_select_div");
					if (element != null) {
						handleEditSelectSingle(parameter.name, getParameterValues(parameter), getParameterLabels(parameter), element);
					}
				}
        	} else {
        		updateLabel(parameter);
        	}
        },
        myErrorFunction: function(errorType, errorMessage) {
        	alert("Error: " + errorType.message);
        }
    };
    callRestService("getParameterValues", 
        {reportUUID: reportUUID, parameter: parameter.name, showArchived: showArchived}, obj, 
        "mySuccessFunction", "myErrorFunction");
}

// Given parameterName (the name of one of the special parameters), and
// pre-supplied parameterValue, store the parameterValue in the variable
// for later consumption, and propagate to any children if this is part
// of a cascading parameter group.
function initializeSpecialParameter(parameterName) {
	// Find the pre-supplied value, which is stored in a variable
	var parameterValue = null;
    if (parameterName.indexOf("ProjectAreaName") == 0) {
    	parameterValue = CURRENT_PROJECT_AREA;
    }
    if (parameterName.indexOf("ProjectAreaItemId") == 0) {
    	parameterValue = CURRENT_PROJECT_AREA;
    }
    // Find the index of the parameter in the allParameterList.
    var i = getAllParameterIndex(parameterName);

    // Store the value
    allParameterValues[i] = parameterValue;

    // Initialize a child parameter, if we're part of a group.
    var group = findGroup(parameterName);
    if (group != null) {
	    // Find next parameter name
	    var childParameter = findNextParameter(group, parameterName);
	    if (isSpecialParameter(childParameter.name)) {
	    	initializeSpecialParameter(childParameter.name);
	    }
	}
}

/* Support for switching to edit mode and recreating parameter widgets */

// Switch to edit mode.
function edit() {
	if (inEditMode == true) return;
	// Just in case, trigger a mouse over.
	var div = document.getElementById("parameterOuterDiv");
	mouseOverDiv(div);

	inEditMode = true;
	
	// Get rid of the helpful tooltip
	div.title = "";
	
	// Destroy any lingering Dojo widgets from other report pages
	var f = function(widget) {
		if (widget.id != null && (typeof(widget.id) == "string")) {
			if (widget.id.indexOf("ReportsDojoWidget") == 0) {
				widget.destroy();
			}
		}
	};
	dijit.registry.forEach(f);
	
	// Show the Run | Cancel buttons
	var actionDiv = document.getElementById("parameterActionDiv");
	actionDiv.innerHTML = "<a class='link' href='#' id='topRun' onclick='return false;'>Run</a><a class='separator'>|</a><a class='link' href='#' id='topCancel' onclick='return false;'>Cancel</a>";
	var bottomActionOuterDiv = document.getElementById("bottomActionOuterDiv");
	bottomActionOuterDiv.className = "visibleBottomActionOuterDiv";
	var bottomActionDiv = document.getElementById("bottomActionDiv");
	bottomActionDiv.innerHTML = "<a class='link' href='#' id='bottomRun' onclick='return false;'>Run</a><a class='separator'>|</a><a class='link' href='#' id='bottomCancel' onclick='return false;'>Cancel</a>";

	// Open the twisty
	var twisty = document.getElementById("parameterTwisty");
	twisty.src = urlPrefix + "reports/resources/app/internal/images/twistie-open.gif";

	// Hook the run events
	var runLink = document.getElementById("topRun");
	if (isIe()) {
		runLink.attachEvent("onclick", handleRun);
	} else {
		runLink.addEventListener("click", handleRun, false);
	}
	runLink = document.getElementById("bottomRun");
	if (isIe()) {
		runLink.attachEvent("onclick", handleRun);
	} else {
		runLink.addEventListener("click", handleRun, false);
	}
	
	// Hook the cancel events
	var cancelLink = document.getElementById("topCancel");
	if (isIe()) {
		cancelLink.attachEvent("onclick", handleCancel);
	} else {
		cancelLink.addEventListener("click", handleCancel, false);
	}
	cancelLink = document.getElementById("bottomCancel");
	if (isIe()) {
		cancelLink.attachEvent("onclick", handleCancel);
	} else {
		cancelLink.addEventListener("click", handleCancel, false);
	}
	
	for (var i = 0; i < parameterList.length; i++) {
        var parameter = parameterList[i];
        if (parameter.isGroup == true) {
            editParameterGroup(parameter);
        } else if (isSpecialParameter(parameter.name)) {
        	// Nothing to do for a hidden parameter
        } else {
            editNonCascadingParameter(parameter);
        }
	}
	// Return false so the event is not propagated
	return false;
}

function editParameterGroup(group) {
	for (var i = 0; i < group.children.length; i++) {
		var child = group.children[i];
		if (isSpecialParameter(child.name)) {
	    	initializeSpecialParameter(child.name);
	    } else {
			// Swap out the label for an editable widget depending on the type
			var widget = document.getElementById(child.name + "_select_div");
			if (widget != null) {
				// Fetch the values to populate the list with
				handleEditSelectSingle(child.name, getParameterValues(child), getParameterLabels(child), widget);
			} else {
				widget = document.getElementById(child.name + "_checkboxList_div");
				if (widget != null) {
					handleEditCheckboxList(child.name, getParameterValues(child), getParameterLabels(child), widget);
				} else {
					widget = document.getElementById(child.name + "_checkbox_div");
					if (widget != null) {
						handleEditCheckbox(child.name, widget);
					} else {
						widget = document.getElementById(child.name + "_text_div");
						if (widget != null) {
							handleEditText(child.name, widget);
						} else {
							widget = document.getElementById(child.name + "_date_div");
							if (widget != null) {
								handleEditDate(child.name, widget);
							} else {
								alert("Unknown widget type " + widget);
							}
						}
					}
				}
			}
	    }
	}
}

function editNonCascadingParameter(parameter) {
	// Swap out the label for an editable widget depending on the type
	var widget = document.getElementById(parameter.name + "_select_div");
	if (widget != null) {
		handleEditSelectSingle(parameter.name, getParameterValues(parameter), getParameterLabels(parameter), widget);
	} else {
		widget = document.getElementById(parameter.name + "_checkboxList_div");
		if (widget != null) {
			handleEditCheckboxList(parameter.name, getParameterValues(parameter), getParameterLabels(parameter), widget);
		} else {
			widget = document.getElementById(parameter.name + "_checkbox_div");
			if (widget != null) {
				handleEditCheckbox(parameter.name, widget);
			} else {
				widget = document.getElementById(parameter.name + "_text_div");
				if (widget != null) {
					handleEditText(parameter.name, widget);
				} else {
					widget = document.getElementById(parameter.name + "_date_div");
					if (widget != null) {
						handleEditDate(parameter.name, widget);
					} else {
						alert("Unknown widget type " + widget);
					}
				}
			}
		}
	}
}

// Create the editable widgets for the given parameter, a single-select combo,
// and insert it into the given element.
function handleEditSelectSingle(parameterName, values, labels, element) {
	if (values == null || values == undefined) {
		element.innerHTML = "<table cellspacing='0px'><tr><td class='tightTd'>" +
			"<div id='" + escapeQuotes(parameterName) + "_label' class='editableSelect'>" +
			"Loading...</div></td></tr><tr><td class='tightTd'><div class='comboDiv'>" +
			"<select size='10' id='" + escapeQuotes(parameterName) + "_select' class='singleSelectCombo'>" +
			"</select></div></td></tr></table>";
		fixComboWidth();
		return;
	}
	
	// Remove duplicates from the list
    var newValues = values;
    var newLabels = labels;
    var noDups = removeDuplicatesWithLabels(newValues, newLabels);
    newValues = noDups.values;
    newLabels = noDups.labels;

	// Sort the list		        
    var sorted = sort(newLabels, newValues);
    newValues = sorted.values;
    newLabels = sorted.labels;
    
    // Get the current value
    var parameterIndex = getAllParameterIndex(parameterName);
	var currentValue = allParameterValues[parameterIndex];
	var flags = allParameterFlags[parameterIndex];
	
	var innerHTML = "<table cellspacing='0px'><tr><td class='tightTd'>" +
		"<div id='" + escapeQuotes(parameterName) + "_label' class='editableSelect'>" +
		getLabelFor(getCurrentOrDefaultLabels(allParameterList[parameterIndex])) + "</div></td></tr><tr><td class='tightTd'><div class='comboDiv'>" +
		"<select size='10' id='" + escapeQuotes(parameterName) + "_select' class='singleSelectCombo' onChange='singleComboSelected(this)'>";	
			
	// Insert a special "No Selection" option
	innerHTML += "<option>(No selection)</option>";	
	for (var i = 0; i < newValues.length; i++) {
		// Quick hack for single-select iterations.
		if ((((flags & ITERATION_NAME) == ITERATION_NAME) || ((flags & ITERATION_ITEMID) == ITERATION_ITEMID)) && (newValues[i] == CURRENT_ITERATION)) {
			// Skip it
			continue;
		}
	
		innerHTML += "<option value='" + escapeQuotes(newValues[i]) + "'";
		if (currentValue == newValues[i]) {
			innerHTML += " selected='true'";
		}
		// If there is a default value and no currentValue, and this is the default value, check it.
		if (currentValue == null && allParameterDefaultValue[parameterIndex] == newValues[i]) {
			innerHTML += " selected='true'";
		}
		innerHTML += ">" + newLabels[i] + "</option>";
	}
	innerHTML += "</select></div></td></tr></table>";
		
	element.innerHTML = innerHTML;
	fixComboWidth();
}

// Create the editable widgets for the given parameter, a multi-select combo,
// and insert it into the given element.
function handleEditCheckboxList(parameterName, values, labels, element) {
	// Create (or re-create) the contents of the list
	updateCheckboxListContents(parameterName, values, labels, element);
}

function updateSelectedIndicator(parameterName) {
	var selectedIndicatorTd = document.getElementById(parameterName + "_selectedIndicator");
	selectedIndicatorTd.innerHTML = "<span class='selectedIndicator'>" + getSelectedIndicatorLabel(parameterName) + "</span>";
}

// If the checkbox list for the given parameter exists and is already
// in edit mode, update its contents for the given values.
function updateCheckboxListContents(parameterName, values, labels, element) {
	var escapedParameterName = escapeQuotes(parameterName);
	// TODO: should this method also clear out the allParametersValue entry
	// for this parameter?
	// TODO: Preserve selection wherever possible?
	// Check to see if the parameter values have come back yet
	if (values == null || values == undefined) {
		element.innerHTML = "<form>" + 
			"<table cellpadding='0px' cellspacing='0px' width='100%'><tr><td>" +
			"</td><td align=right class='selectDeselectTd' id='" + escapedParameterName + "_selectDeselect'>" +
			"<a class='selectDeselectLink' href='#' onclick='return selectAll(\"" + parameterName + "\");'>Select All</a><a class='separator'>|</a><a class='selectDeselectLink' href='#' onclick='return deselectAll(\"" + parameterName + "\");'>Deselect All</a>" +			"</td></tr></table>" +
			"<ul id='loadingList_'" + escapedParameterName + "' class='multiCheckboxList'><li><label id='" + escapedParameterName + "_loadingLabel' for='" + escapedParameterName + "_loading'>" +
			"<input class='multiCheckboxListInput' id='" + escapedParameterName + "_loading' type='checkbox' disabled='true'/>Loading...</label></li></ul></form>";
		var label = document.getElementById(parameterName + "_loadingLabel");
		label.onmouseover = function() {this.className = "hover"; };
		label.onmouseout = function() { this.className = ""; };
		fixListHeight('loadingList_' + parameterName);
		// Update the "0 of 10 selected" label
		updateSelectedIndicator(parameterName);
		return;
	}
	// Remove duplicates from the list
    var newValues = values;
    var newLabels = labels;
    var noDups = removeDuplicatesWithLabels(newValues, newLabels);
    newValues = noDups.values;
    newLabels = noDups.labels;

	// Sort the list		        
    var sorted = sortTree(newLabels, newValues);
    newValues = sorted.values;
    newLabels = sorted.labels;
    
    // Get the current values
    var parameterIndex = getAllParameterIndex(parameterName);
	var currentValues = allParameterValues[parameterIndex];
	// Use default values if there are no current values
	if (currentValues == null) {
		currentValues = allParameterDefaultValue[parameterIndex];
	}
	if (!(currentValues instanceof Array)) {
		currentValues = [currentValues];
	}
	
	var innerHTML = null;
	// Check if this is a special widget
	var flags = allParameterFlags[parameterIndex];
	if (((flags & CONTRIBUTOR_NAME) == CONTRIBUTOR_NAME) || ((flags & CONTRIBUTOR_USERID) == CONTRIBUTOR_USERID)) {
		var currentContributorIndex = 0;
		var foundCurrentContributor = false;
		for (var i = newValues.length - 1; i >= 0; i--) {
			if (newValues[i] == CURRENT_CONTRIBUTOR) {
				currentContributorIndex = i;
				foundCurrentContributor = true;
				break;
			}
		}
		
		innerHTML = "<form>";
		innerHTML += "<table cellpadding='0px' cellspacing='0px' width='100%'><tr><td class='selectDeselectTd'>";
		innerHTML += "<div class='decorativeCheckboxDiv'><input type='checkbox'";
		if (!foundCurrentContributor) {
			innerHTML += " disabled";
		}
		innerHTML += " id='" + escapedParameterName + "_input" + currentContributorIndex + "' class='specialCheckbox' name='" + CURRENT_CONTRIBUTOR + "' onclick='checkboxListItemSelected(this)' ";
		if (contains(currentValues, CURRENT_CONTRIBUTOR) && foundCurrentContributor) {
			innerHTML += " checked='true'";
		}
		innerHTML += "></div></td>";
		innerHTML += "<td class='selectDeselectTd'>";
		if (foundCurrentContributor) {
			innerHTML += "<span class='decorativeCheckboxSpan'>";
		} else {
			innerHTML += "<span class='decorativeCheckboxSpanDisabled'>";
		}
		innerHTML += "Current Contributor</span></td>";
		innerHTML += "</input></div>";
		innerHTML += "</td><td align=right class='selectDeselectTd' id='" + escapedParameterName + "_selectDeselect'>";
		innerHTML += "<a class='selectDeselectLink' href='#' onclick='return selectAll(\"" + parameterName + "\");'>Select All</a><a class='separator'>|</a><a class='selectDeselectLink' href='#' onclick='return deselectAll(\"" + parameterName + "\");'>Deselect All</a>";
		innerHTML += "</td></tr></table>";
		innerHTML += "<ul id='checkboxList_" + parameterName + "' class='multiCheckboxList'>";
		for (var i = 0; i < newValues.length; i++) {
			if (newValues[i] == CURRENT_CONTRIBUTOR) {
				// Skip it
				continue;
			}
			innerHTML += "<li><label id='" + escapedParameterName + "_checkboxLabel" + i + "' for='" + escapedParameterName + "_input" + i + "'";
			innerHTML += " style='padding-left: " + (10 + (15 * treeLevel(newLabels[i]))) + "px'";
			innerHTML += "><input class='multiCheckboxListInput' id='" + escapedParameterName + "_input" + i + "' name='";
			innerHTML += escapeQuotes(newValues[i]);
			innerHTML += "' type='checkbox'";
			if (contains(currentValues, newValues[i])) {
				innerHTML += " checked='true'";
			}
			// If there is a default value and no currentValue, and this is the default value, check it.
			if (currentValues.length == 0 && allParameterDefaultValue[parameterIndex] == newValues[i]) {
				innerHTML += " checked='true'";
			}
			innerHTML += " onclick='checkboxListItemSelected(this)'/>";
			innerHTML += stringToHTML(treeString(newLabels[i]));
			innerHTML += "</label></li>";
		}
		innerHTML += "</ul></form>";
	} else if (((flags & ITERATION_NAME) == ITERATION_NAME) || ((flags & ITERATION_ITEMID) == ITERATION_ITEMID)) {
		var currentIterationIndex = 0;
		var foundCurrentIteration = false;
		for (var i = newValues.length - 1; i >= 0; i--) {
			if (newValues[i] == CURRENT_ITERATION) {
				currentIterationIndex = i;
				foundCurrentIteration = true;
				break;
			}
		}
		
		innerHTML = "<form>";
		innerHTML += "<table cellpadding='0px' cellspacing='0px' width='100%'><tr><td class='selectDeselectTd'>";
		innerHTML += "<div class='decorativeCheckboxDiv'><input type='checkbox'";
		if (!foundCurrentIteration) {
			innerHTML += " disabled";
		}
		innerHTML += " id='" + escapedParameterName + "_input" + currentIterationIndex + "' class='specialCheckbox' name='" + CURRENT_ITERATION + "' onclick='checkboxListItemSelected(this)' ";
		if (contains(currentValues, CURRENT_ITERATION) && foundCurrentIteration) {
			innerHTML += " checked='true'";
		}
		innerHTML += "></div></td>";
		innerHTML += "<td class='selectDeselectTd'>";
		if (foundCurrentIteration) {
			innerHTML += "<span class='decorativeCheckboxSpan'>";
		} else {
			innerHTML += "<span class='decorativeCheckboxSpanDisabled'>";
		}
		innerHTML += "Current Iteration</span></td>";
		innerHTML += "</input></div>";
		innerHTML += "</td><td align=right class='selectDeselectTd' id='" + escapedParameterName + "_selectDeselect'>";
		innerHTML += "<a class='selectDeselectLink' href='#' onclick='return selectAll(\"" + parameterName + "\");'>Select All</a><a class='separator'>|</a><a class='selectDeselectLink' href='#' onclick='return deselectAll(\"" + parameterName + "\");'>Deselect All</a>";
		innerHTML += "</td></tr></table>";
		innerHTML += "<ul id='checkboxList_" + parameterName + "' class='multiCheckboxList'>";
		for (var i = 0; i < newValues.length; i++) {
			if (newValues[i] == CURRENT_ITERATION) {
				// Skip it
				continue;
			}
			innerHTML += "<li><label id='" + escapedParameterName + "_checkboxLabel" + i + "' for='" + escapedParameterName + "_input" + i + "'";
			innerHTML += " style='padding-left: " + (10 + (15 * treeLevel(newLabels[i]))) + "px'";
			innerHTML += "><input class='multiCheckboxListInput' id='" + escapedParameterName + "_input" + i + "' name='";
			innerHTML += escapeQuotes(newValues[i]);
			innerHTML += "' type='checkbox'";
			if (contains(currentValues, newValues[i])) {
				innerHTML += " checked='true'";
			}
			// If there is a default value and no currentValue, and this is the default value, check it.
			if (currentValues.length == 0 && allParameterDefaultValue[parameterIndex] == newValues[i]) {
				innerHTML += " checked='true'";
			}
			innerHTML += " onclick='checkboxListItemSelected(this)'/>";
			innerHTML += stringToHTML(treeString(newLabels[i]));
			innerHTML += "</label></li>";
		}
		innerHTML += "</ul></form>";
	} else if (((flags & TEAM_AREA_NAME) == TEAM_AREA_NAME) || ((flags & TEAM_AREA_ITEMID) == TEAM_AREA_ITEMID)) {
		var tree = createTree(newValues, newLabels);
		var myTeamAreasIndex = 0;
		var foundMyTeamAreas = false;
		for (var i = newValues.length - 1; i >= 0; i--) {
			if (newValues[i] == MY_TEAM_AREAS) {
				myTeamAreasIndex = i;
				foundMyTeamAreas = true;
				break;
			}
		}
		
		innerHTML = "<form>";
		innerHTML += "<table cellpadding='0px' cellspacing='0px' width='100%'><tr><td class='selectDeselectTd'>";
		innerHTML += "<div class='decorativeCheckboxDiv'><input type='checkbox'";
		if (!foundMyTeamAreas) {
			innerHTML += " disabled";
		}
		innerHTML += " id='" + escapedParameterName + "_input" + myTeamAreasIndex + "' class='specialCheckbox' name='" + MY_TEAM_AREAS + "' onclick='checkboxListItemSelected(this)' ";
		if (contains(currentValues, MY_TEAM_AREAS) && foundMyTeamAreas) {
			innerHTML += " checked='true'";
		}
		innerHTML += "></div></td>";
		innerHTML += "<td class='selectDeselectTd'>";
		if (foundMyTeamAreas) {
			innerHTML += "<span class='decorativeCheckboxSpan'>";
		} else {
			innerHTML += "<span class='decorativeCheckboxSpanDisabled'>";
		}
		innerHTML += "My Team Areas</span></td>";
		innerHTML += "</input></div>";
		innerHTML += "</td><td align=right class='selectDeselectTd' id='" + escapedParameterName + "_selectDeselect'>";
		innerHTML += "<a class='selectDeselectLink' href='#' onclick='return selectAll(\"" + parameterName + "\");'>Select All</a><a class='separator'>|</a><a class='selectDeselectLink' href='#' onclick='return deselectAll(\"" + parameterName + "\");'>Deselect All</a>";
		innerHTML += "</td></tr></table>";
		innerHTML += "<ul id='checkboxList_" + parameterName + "' class='multiCheckboxList'>";
		var callbackFunction = function(label, value, level, fabricated, id){
			if (value == MY_TEAM_AREAS) {
				// Skip it
				return;
			}
			innerHTML += "<li><label id='" + escapedParameterName + "_checkboxLabel" + id + "' for='" + escapedParameterName + "_input" + id + "'";
			innerHTML += " style='padding-left: " + (10 + (15 * level)) + "px;";
			if (fabricated == true) {
				innerHTML += " color: #999999;";
			}
			innerHTML += "'><input class='multiCheckboxListInput' id='" + escapedParameterName + "_input" + id + "' name='";
			innerHTML += escapeQuotes((value == null ? label : value));
			innerHTML += "' type='checkbox'";
			if (fabricated == true) {
				innerHTML += " disabled";
			}
			// If this value is in currentValues, check it.
			if (value != null && contains(currentValues, value)) {
				innerHTML += " checked='true'";
			}
			// If there is a default value and no currentValue, and this is the default value, check it.
			if (value != null && currentValues.length == 0 && allParameterDefaultValue[parameterIndex] == value) {
				innerHTML += " checked='true'";
			}
			innerHTML += " onclick='checkboxListItemSelected(this)'/>";
			innerHTML += stringToHTML(label);
			innerHTML += "</label></li>";
		};
		visit(tree, callbackFunction, 0, myTeamAreasIndex);
		innerHTML += "</ul></form>";
	} else if ((flags & HIERARCHICAL) == HIERARCHICAL) {
		var tree = createTree(newValues, newLabels);
		innerHTML = "<form>";
		innerHTML += "<table cellpadding='0px' cellspacing='0px' width='100%'><tr><td>";
		innerHTML += "</td><td align=right class='selectDeselectTd' id='" + escapedParameterName + "_selectDeselect'>";
		innerHTML += "<a class='selectDeselectLink' href='#' onclick='return selectAll(\"" + parameterName + "\");'>Select All</a><a class='separator'>|</a><a class='selectDeselectLink' href='#' onclick='return deselectAll(\"" + parameterName + "\");'>Deselect All</a>";
		innerHTML += "</td></tr></table>";
		innerHTML += "<ul id='checkboxList_" + parameterName + "' class='multiCheckboxList'>";
		var callbackFunction = function(label, value, level, fabricated, id){
			innerHTML += "<li><label id='" + escapedParameterName + "_checkboxLabel" + id + "' for='" + escapedParameterName + "_input" + id + "'";
			innerHTML += " style='padding-left: " + (10 + (15 * level)) + "px;";
			if (fabricated == true) {
				innerHTML += " color: #999999;";
			}
			innerHTML += "'><input class='multiCheckboxListInput' id='" + escapedParameterName + "_input" + id + "' name='";
			innerHTML += escapeQuotes((value == null ? label : value));
			innerHTML += "' type='checkbox'";
			if (fabricated == true) {
				innerHTML += " disabled";
			}
			// If this value is in currentValues, check it.
			if (value != null && contains(currentValues, value)) {
				innerHTML += " checked='true'";
			}
			// If there is a default value and no currentValue, and this is the default value, check it.
			if (value != null && currentValues.length == 0 && allParameterDefaultValue[parameterIndex] == value) {
				innerHTML += " checked='true'";
			}
			innerHTML += " onclick='checkboxListItemSelected(this)'/>";
			innerHTML += stringToHTML(label);
			innerHTML += "</label></li>";
		}
		visit(tree, callbackFunction, 0, -1);
		innerHTML += "</ul></form>";
	} else {
		innerHTML = "<form>";
		innerHTML += "<table cellpadding='0px' cellspacing='0px' width='100%'><tr><td>";
		innerHTML += "</td><td align=right class='selectDeselectTd' id='" + escapedParameterName + "_selectDeselect'>";
		innerHTML += "<a class='selectDeselectLink' href='#' onclick='return selectAll(\"" + parameterName + "\");'>Select All</a><a class='separator'>|</a><a class='selectDeselectLink' href='#' onclick='return deselectAll(\"" + parameterName + "\");'>Deselect All</a>";
		innerHTML += "</td></tr></table>";
		innerHTML += "<ul id='checkboxList_" + parameterName + "' class='multiCheckboxList'>";
		for (var i = 0; i < newValues.length; i++) {
			innerHTML += "<li><label id='" + escapedParameterName + "_checkboxLabel" + i + "' for='" + escapedParameterName + "_input" + i + "'";
			innerHTML += " style='padding-left: " + (10 + (15 * treeLevel(newValues[i]))) + "px'";
			innerHTML += "><input class='multiCheckboxListInput' id='" + escapedParameterName + "_input" + i + "' name='";
			innerHTML += escapeQuotes(newValues[i]);
			innerHTML += "' type='checkbox'";
			// If this value is in currentValues, check it.
			if (contains(currentValues, newValues[i])) {
				innerHTML += " checked='true'";
			}
			// If there is a default value and no currentValue, and this is the default value, check it.
			if (currentValues.length == 0 && allParameterDefaultValue[parameterIndex] == newValues[i]) {
				innerHTML += " checked='true'";
			}
			innerHTML += " onclick='checkboxListItemSelected(this)'/>";
			innerHTML += stringToHTML(newLabels[i]);
			innerHTML += "</label></li>";
		}
		innerHTML += "</ul></form>";
	}

	element.innerHTML = innerHTML;
	fixListHeight('checkboxList_' + parameterName);
	// Some manual hover code for IE
	for (var i = 0; i < newValues.length; i++) {
		var label = document.getElementById(parameterName + "_checkboxLabel" + i);
		if (label != null) {
			label.onmouseover = function() { this.className = "hover"; };
			label.onmouseout = function() { this.className = ""; };
		}
	}
	// Update the "0 of 10 selected" label
	updateSelectedIndicator(parameterName);
}

// Create the editable widgets for the given parameter, a single-select combo / checkbox,
// and insert it into the given element.
function handleEditCheckbox(parameterName, element) {
	// The act of editing a checkbox causes a value to be selected, even
	// if none are currently specified.

    // Get the current value
    var parameterIndex = getAllParameterIndex(parameterName);
	var currentValue = allParameterValues[parameterIndex];
	if (currentValue == null) {
		currentValue = allParameterDefaultValue[parameterIndex];
	}
	
	var innerHTML = "<div>" +
		"<select class='booleanCombo' id='" + escapeQuotes(parameterName) + "_select' onChange='checkboxSelected(this)'>" +
		"<option value='true'";
	if (currentValue == "true") {
		innerHTML += " selected='true'";
	}
	innerHTML += ">Yes</option><option value='false'";
	if (currentValue == "false") {
		innerHTML += " selected='true'";
	}
	innerHTML += ">No</option></select></div>";
	element.innerHTML = innerHTML;
	if (currentValue == undefined || currentValue == null) {
		allParameterValues[parameterIndex] = "true";
	}
}

// Create the editable widgets for the given parameter, an editable text field,
// and insert it into the given element.
function handleEditText(parameterName, element) {
    // Get the current value
    var parameterIndex = getAllParameterIndex(parameterName);
	var currentValue = allParameterValues[parameterIndex];
	if (currentValue == undefined || currentValue == null) {
		currentValue = allParameterDefaultValue[parameterIndex]
	}
	
	var type = "text";
	if (allParameterList[parameterIndex].concealed == true) {
		type = "password";
	}
	var innerHTML = "<input id='" + escapeQuotes(parameterName) + "_text' type='" + type + "'  class='text' onChange='textChanged(this)'";
	if (currentValue != undefined && currentValue != null) {
		innerHTML += " value='" + escapeQuotes(currentValue) + "'";
	}
	innerHTML += "/>";
	element.innerHTML = innerHTML;
}

// Create a dojo widget for the given parameter, a date, and insert
// it into the given element.
function handleEditDate(parameterName, element) {
    // Get the current value
    var parameterIndex = getAllParameterIndex(parameterName);
	var currentValue = allParameterValues[parameterIndex];
	var innerHTML = "<input id='" + escapeQuotes(parameterName) + "_date' type='text' dojoType='dijit.form.DateTextBox' class='dateTimeBox'/>";
	element.innerHTML = innerHTML;
	// Now create the widget
	var input = element.childNodes[0];
	if (input != null) {
		var widgetId = "ReportsDojoWidget." + ((queryUUID != null) ? queryUUID : reportUUID) + "." + parameterName;
        var w = new dijit.form.DateTextBox({name: "foo", id: widgetId}, input);
        if (currentValue != null) {
        	w.setValue(currentValue);
        }
		dojo.connect(w, "onChange", null,
			function(e) {
				setDirty(true);
			    // Find the index of the parameter in the allParameterList.
			    var parameterIndex = getAllParameterIndex(parameterName);
			    allParameterValues[parameterIndex] = w.getValue();
			    allParametersIsValueSet[parameterIndex] = true;
			}
		);
        activeDojoWidgets[activeDojoWidgets.length] = w;
		// Date widget dynamically replaced its class so need to set element style.
		w.domNode.style.width = "10.8em";
	}
}

/* Selection/modification handlers for various parameter widget types */

// A single-select combo has been modified. Update the label.
function singleComboSelected(combo) {
	setDirty(true);
	// Strip off the _select suffix
	var parameterName = combo.id.substr(0, combo.id.length - 7);
	// Find the label
	var label = document.getElementById(parameterName + "_label");
	label.childNodes[0].nodeValue = combo.options[combo.selectedIndex].text;
	
    // Find the index of the parameter in the allParameterList.
    var parameterIndex = getAllParameterIndex(parameterName);
    if (combo.selectedIndex == 0) {
        allParameterValues[parameterIndex] = null;
    } else {
	    allParameterValues[parameterIndex] = combo.options[combo.selectedIndex].value;
	}
    allParametersIsValueSet[parameterIndex] = true;
    // If the parameter was in a cascading group, trigger repopulation
    // of all subsequent widgets in the group
    var group = findGroup(parameterName);
    if (group != null) {
	    // Find next parameter name
	    var nextParameter = findNextParameter(group, parameterName);
	    while (nextParameter != null) {
		    if (nextParameter != null) {
			    var element = document.getElementById(nextParameter.name + "_checkboxList_div");
			    	var nextIndex = getAllParameterIndex(nextParameter.name);
				    if (element != null) {
				    	allParameterValues[nextIndex] = null;
				    	allParametersIsValueSet[nextIndex] = true;
					    updateCheckboxListContents(nextParameter.name, getParameterValues(nextParameter), getParameterLabels(nextParameter), element);
					} else {
						element = document.getElementById(nextParameter.name + "_select_div");
						if (element != null) {
					    	allParameterValues[nextIndex] = null;
					    	allParametersIsValueSet[nextIndex] = true;
							handleEditSelectSingle(nextParameter.name, getParameterValues(nextParameter), getParameterLabels(nextParameter), element);
						}
					}
			}
			nextParameter = findNextParameter(group, nextParameter.name);
		}
	}
}
		
// A single-select combo masquerading as a checkbox has been modified.
function checkboxSelected(combo) {
	setDirty(true);
	// Strip off the _select suffix
	var parameterName = combo.id.substr(0, combo.id.length - 7);
	
    // Find the index of the parameter in the allParameterList.
    var parameterIndex = getAllParameterIndex(parameterName);
	allParameterValues[parameterIndex] = combo.options[combo.selectedIndex].value;
	allParametersIsValueSet[parameterIndex] = true;
	// The parameter could not have been part of a cascading group.
}

// A text widget's value has changed.
function textChanged(text) {
	setDirty(true);
    // Strip off the _text suffix
    var parameterName = text.id.substr(0, text.id.length - 5);
    // Find the index of the parameter in the allParameterList.
    var parameterIndex = getAllParameterIndex(parameterName);
    allParameterValues[parameterIndex] = text.value;
    allParametersIsValueSet[parameterIndex] = true;
    // The parameter could not have been part of a cascading group.
}

function checkboxListItemSelected(checkbox) {
	// The value of one checkbox in a checkbox list has changed.
	var parameterName = checkbox.id.substr(0, checkbox.id.indexOf("_input"));
	checkboxListSelected(parameterName);
}

function checkboxListSelected(parameterName) {
	setDirty(true);
	var i = 0;
	var selectedValues = [];
	var selectedIndex = 0;
	var input = document.getElementById(parameterName + "_input" + i);
	while (input != null) {
		if (input.checked == true) {
			selectedValues[selectedIndex] = input.name;
			selectedIndex++;
		}
		i++;
		input = document.getElementById(parameterName + "_input" + i);
	}
    if (selectedValues.length == 1) {
    	selectedValues = selectedValues[0];
    }
    // Find the index of the parameter in the allParameterList.
    var parameterIndex = getAllParameterIndex(parameterName);
    allParameterValues[parameterIndex] = selectedValues;
    allParametersIsValueSet[parameterIndex] = true;
    
    updateSelectedIndicator(parameterName);
    
    // If the parameter was in a cascading group, trigger repopulation
    // of all subsequent widgets in the group
    var group = findGroup(parameterName);
    if (group != null) {
	    // Find next parameter name
	    var nextParameter = findNextParameter(group, parameterName);
	    while (nextParameter != null) {
		    var element = document.getElementById(nextParameter.name + "_checkboxList_div");
		    var nextIndex = getAllParameterIndex(nextParameter.name);
		    if (element != null) {
		    	allParameterValues[nextIndex] = null;
			    allParametersIsValueSet[nextIndex] = true;
			    updateCheckboxListContents(nextParameter.name, getParameterValues(nextParameter), getParameterLabels(nextParameter), element);
			} else {
				element = document.getElementById(nextParameter.name + "_select_div");
				if (element != null) {
			    	allParameterValues[nextIndex] = null;
				    allParametersIsValueSet[nextIndex] = true;
					handleEditSelectSingle(nextParameter.name, getParameterValues(nextParameter), getParameterLabels(nextParameter), element);
				}
			}
			nextParameter = findNextParameter(group, nextParameter.name);
		}
	}
}


/* Various user-initiated actions */

function selectAll(parameterName) {
	var i = 0;
	while (true) {
		var input = document.getElementById(parameterName + "_input" + i);
		if (input == null) break;
		if (input.disabled != true) {
			input.checked = true;
		}
		i++;
	}
	// Propagate to children
	checkboxListSelected(parameterName);
	return false;
}

function deselectAll(parameterName) {
	var i = 0;
	while (true) {
		var input = document.getElementById(parameterName + "_input" + i);
		if (input == null) break;
		if (input.disabled != true) {
			input.checked = false;
		}
		i++;
	}
	// Propagate to children
	checkboxListSelected(parameterName);
	return false;
}
		
function doSetAsDefault() {
	var postArgs = {queryUUID: queryUUID};
    var obj = {
        mySuccessFunction: function(args) {
        	alert("This is now the default report.");
			postSetAsDefault(postArgs);
        },
        myErrorFunction: function(errorType, errorMessage) {
        	alert("Error: " + errorType.message);
        }
    };

    var args = {
    	queryUUID: queryUUID
    };
    callRestService("postSetDefaultQueryDescriptor", 
        args, obj, 
        "mySuccessFunction", "myErrorFunction");
}

function openExportDialog() {
    if (!hasAllRequiredParameters()) {
        alert("Please enter values for all required parameters");
        return;
    }

	// Centre and show the dialog
	var floatingPane = document.getElementById("exportFloatingPane");
	// TODO: Don't hardcode this
	//floatingPane.style.top = "" + ((window.innerHeight - floatingPane.style.height) / 2) + "px";
	//floatingPane.style.left = "" + (window.innerWidth - floatingPane.style.width) / 2 + "px";
	// Get the inner height and width in a way supported by all browsers
	var innerHeight = window.innerHeight ? window.innerHeight : Math.max(document.body.clientHeight, document.documentElement.clientHeight);
	var innerWidth = window.innerWidth ? window.innerWidth : Math.max(document.body.clientWidth, document.documentElement.clientWidth);
	
	floatingPane.style.top = "" + ((innerHeight - 300) / 2) + "px";
	floatingPane.style.left = "" + ((innerWidth - 70) / 2) + "px";
	floatingPane.style.visibility = "visible";
}

function doExport() {
	// Get the value from the dialog
	var select = document.getElementById("exportCombo");
	var format = select.options[select.selectedIndex].value;
	
	// Hide the Export panel
	var floatingPane = document.getElementById("exportFloatingPane");
	floatingPane.style.visibility = "hidden";
	
    if (!hasAllRequiredParameters()) {
        alert("Please enter values for all required parameters");
        return;
    }
        
	var newParameters = prepareParameters(allParameterValues);    
        
    // Instead of calling the REST service, we'll call the ReportViewerService.
    var url = urlPrefix + "service/com.ibm.team.reports.service.internal.IReportViewerService";
    
	url += "?__format=" + format;
	url += "&__reportUUID=" + reportUUID;
	for (var i = 0; i < newParameters.names.length; i++) {
		url += "&" + escape(newParameters.names[i]);
		url += "=" + escape(newParameters.values[i]);
	}
    document.location.href = url;
}

function saveAs() {
	// Null out any leftover folder and name values
	var nameWidget = document.getElementById("saveAsName");
	if (nameWidget != null) {
		nameWidget.value = "";
	}
	// TODO: Null out the folder

	// Centre and show the dialog
	var floatingPane = document.getElementById("floatingPane");
	// TODO: Don't hardcode this
	//floatingPane.style.top = "" + ((window.innerHeight - floatingPane.style.height) / 2) + "px";
	//floatingPane.style.left = "" + (window.innerWidth - floatingPane.style.width) / 2 + "px";
	// Get the inner height and width in a way supported by all browsers
	var innerHeight = window.innerHeight ? window.innerHeight : Math.max(document.body.clientHeight, document.documentElement.clientHeight);
	var innerWidth = window.innerWidth ? window.innerWidth : Math.max(document.body.clientWidth, document.documentElement.clientWidth);
	
	floatingPane.style.top = "" + ((innerHeight - 300) / 2) + "px";
	floatingPane.style.left = "" + ((innerWidth - 70) / 2) + "px";
	floatingPane.style.visibility = "visible";
}

function cancelSaveAs() {
	// Hide the dialog
	var floatingPane = document.getElementById("floatingPane");
	floatingPane.style.visibility = "hidden";
}

function cancelExport() {
	// Hide the dialog
	var floatingPane = document.getElementById("exportFloatingPane");
	floatingPane.style.visibility = "hidden";
}


function save() {
    var obj = {
        mySuccessFunction: function(args) {
		    setDirty(false);
			postSave();
            // TODO: set the new query UUID and any other interesting fields
        },
        myErrorFunction: function(errorType, errorMessage) {
        	alert("Error: " + errorType.message);
        }
    };

	var newParameters = prepareParameters(allParameterValues);    

    var args = {
    	queryUUID: queryUUID,
        reportUUID: reportUUID,
        folderUUID: queryFolderUUID,
        parameterName: newParameters.names,
        parameterValue: newParameters.values
    };
    callRestService("postSaveQueryDescriptor", 
        args, obj, 
        "mySuccessFunction", "myErrorFunction");
    
    // Set the initial parameter values to the new ones
    for (var i = 0; i < allParameterValues.length; i++) {
    	initialParameterValues[i] = allParameterValues[i];
    }
}

function doSaveAs() {
    var obj = {
        mySuccessFunction: function(args) {
		    setDirty(false);
			postSave();
            // TODO: set the new query UUID and any other interesting fields
        },
        myErrorFunction: function(errorType, errorMessage) {
        	alert("Error: " + errorType.message);
        }
    };

	var newParameters = prepareParameters(allParameterValues);    

	var nameWidget = document.getElementById("saveAsName");
	
	if (nameWidget.value == null || nameWidget.value == "") {
		alert("Please enter a name.");
		return;
	}
	
    var args = {
        name: nameWidget.value,
        reportUUID: reportUUID,
        parameterName: newParameters.names,
        parameterValue: newParameters.values
    };
    callRestService("postSaveQueryDescriptor", 
        args, obj, 
        "mySuccessFunction", "myErrorFunction");
	
	// Now that we're done, hide the Save As panel
	var floatingPane = document.getElementById("floatingPane");
	floatingPane.style.visibility = "hidden";
}

function cancel(discardChanges) {
	// Switch back to display mode.
	inEditMode = false;
	if (discardChanges) {
		setDirty(false);
	}
	
	var div = document.getElementById("parameterOuterDiv");
	div.title = "Click to Edit Parameters";
	
	// Remove the Run | Cancel buttons
	var actionDiv = document.getElementById("parameterActionDiv");
	actionDiv.innerHTML = "";
	var bottomActionOuterDiv = document.getElementById("bottomActionOuterDiv");
	bottomActionOuterDiv.className = "hiddenBottomActionOuterDiv";
	// Close the twisty
	var twisty = document.getElementById("parameterTwisty");
	twisty.src = urlPrefix + "reports/resources/app/internal/images/twistie-closed.gif";
	
	var valuesChanged = false;
	// For each parameter, swap out the editable widget for a label depending on the type
	for (var i = 0; i < allParameterList.length; i++) {
		var parameterName = allParameterList[i].name;
		if (isSpecialParameter(parameterName)) continue;
		var widget = document.getElementById(parameterName + "_select_div");
		if (widget == null) {
			widget = document.getElementById(parameterName + "_checkboxList_div");
			if (widget == null) {
				widget = document.getElementById(parameterName + "_checkbox_div");
				if (widget == null) {
					widget = document.getElementById(parameterName + "_text_div");
					if (widget == null) {
						widget = document.getElementById(parameterName + "_date_div");
					}
				}
			} else {
    			// Hide the Select All and Deselect All buttons if they exist
    			// TODO: Don't need this code anymore.
				var selectDeselectTd = document.getElementById(parameterName + "_selectDeselect");
				if (selectDeselectTd != null) {
					selectDeselectTd.innerHTML = "";
				}
				// Hide the "0 of 10 selected" indicator if it exists
				var selectedIndicatorTd = document.getElementById(parameterName + "_selectedIndicator");
				if (selectedIndicatorTd != null) {
					selectedIndicatorTd.innerHTML = "";
				}
			}
		}
		if (widget == null) {
			alert("Unknown widget type " + widget);
			continue;
		}
		
		if (equals(allParameterValues[i], initialParameterValues[i]) == false) {
			valuesChanged = true;
		}
		
		var initialValues = initialParameterValues[i];
		if (initialValues && !(initialValues instanceof Array)) {
			initialValues = [initialValues];
		}
		
		if (discardChanges) {
			// Forget any pending changes
			allParameterValues[i] = initialParameterValues[i];
		    allParametersIsValueSet[i] = initialAllParametersIsValueSet[i];
		}
		
		// Update the labels
		updateLabel(allParameterList[i], true);
		
	}
	if (discardChanges) {
		// Reload the report based on the original parameters so that
		// the displayed report will match the displayed parameters.
		if (valuesChanged == true) {
			updateReport(true, false);
		}
	}
	
	// Destroy any dojo widgets which are disappearing
	for (var i = 0; i < activeDojoWidgets.length; i++) {
		activeDojoWidgets[i].destroy();
	}
	activeDojoWidgets = [];
		
	return false;
}

function handleEdit(event) {
	if (inEditMode == true) return;
	edit();
	// Stop event propagation in all browsers
	event.cancelBubble = true;
	if (event.stopPropagation) event.stopPropagation();
}	

function handleTwisty(event) {
	if (inEditMode == false) return;
	cancel(false);
	// Stop event propagation in all browsers
	event.cancelBubble = true;
	if (event.stopPropagation) event.stopPropagation();
}

function handleCancel(event) {
	if (inEditMode == false) return;
	cancel(true);
	// Stop event propagation in all browsers
	event.cancelBubble = true;
	if (event.stopPropagation) event.stopPropagation();
}	

function handleRun(event) {
	updateReport(false, true);
	cancel(false);
	// Stop event propagation in all browsers
	event.cancelBubble = true;
	if (event.stopPropagation) event.stopPropagation();
}

/* Mouse movement and hover handlers */

function mouseOverDiv(element) {
	// If we're in edit mode, ignore the mouse in.
	if (inEditMode == true) return;
	element.className = "hoverParameterOuterDiv";
	var parameterDiv = document.getElementById("parameterDiv");
	parameterDiv.className = "hoverParameterDiv";
	var twistyLink = document.getElementById("twistyLink");
	twistyLink.className = "twistyHover";
	// Dark twisty
	var twisty = document.getElementById("parameterTwisty");
	twisty.src = urlPrefix + "reports/resources/app/internal/images/twistie-closed.gif";
	
	for (var i = 0; i < allParameterList.length; i++) {
		if (isSpecialParameter(allParameterList[i].name)) continue;
		var name = allParameterList[i].name;
		var label = document.getElementById(name + "_paramLabel");
		if (allParameterAllowNull[i] == false) {
			// Mandatory parameter: add a required marker
			var span = document.getElementById(name + "_labelSpan");
			var label = span.childNodes[0].nodeValue;
			label += "<span class='requiredMarker'>*</span>";
			span.innerHTML = label;
		}
	}
	var node_list = document.getElementsByTagName("span");
	for (var i = 0; i < node_list.length; i++) {
		var span = node_list[i];
		if (span.className == "labelLightText") {
			span.className = "labelDarkText";
		}
		if (span.className == "paramValueLightText") {
			span.className = "paramValueDarkText";
		}
	}
	var div = document.getElementById("topDarkLine");
	div.className = "compactDarkRule";
	div = document.getElementById("bottomDarkLine");
	div.className = "compactDarkRule";
}

function mouseOutDiv(element) {
	// If we're in edit mode, ignore the mouse out.
	if (inEditMode == true) return;
	element.className = "noHoverParameterOuterDiv";
	var parameterDiv = document.getElementById("parameterDiv");
	parameterDiv.className = "noHoverParameterDiv";
	var twistyLink = document.getElementById("twistyLink");
	twistyLink.className = "twistyNoHover";
	// Light twisty
	var twisty = document.getElementById("parameterTwisty");
	twisty.src = urlPrefix + "reports/resources/app/internal/images/twistie-closed-light.gif";
	var node_list = document.getElementsByTagName("span");
	for (var i = 0; i < node_list.length; i++) {
		var span = node_list[i];
		if (span.className == "labelDarkText") {
			span.className = "labelLightText";
			// If the parameter is required, remove the red *
			for (var j = 0; j < span.childNodes.length; j++) {
				if (span.childNodes[j].tagName == "SPAN") {
					span.removeChild(span.childNodes[j]);
				}
			}
		}
		if (span.className == "paramValueDarkText") {
			span.className = "paramValueLightText";
		}
	}
	
	var div = document.getElementById("topDarkLine");
	div.className = "compactInvisibleDarkRule";
	div = document.getElementById("bottomDarkLine");
	div.className = "compactInvisibleDarkRule";
}

/* Support for the Save As... and Export floating dialogs */

function dragHandler(event) {
	var target = document.getElementById("floatingPane");
	savedTarget = target;
	dragOK = true;
	dragXoffset = event.clientX - parseInt(target.style.left);
	dragYoffset = event.clientY - parseInt(target.style.top);
	return false;
}

function exportDragHandler(event) {
	var target = document.getElementById("exportFloatingPane");
	savedTarget = target;
	dragOK = true;
	dragXoffset = event.clientX - parseInt(target.style.left);
	dragYoffset = event.clientY - parseInt(target.style.top);
	return false;
}

function moveHandler(event) {
	if (event == null) event = window.event;
	if (event.button <= 1 && dragOK) {
		savedTarget.style.left = event.clientX - dragXoffset + "px";
		savedTarget.style.top = event.clientY - dragYoffset + "px";
	}
	return false;
}

function endDragHandler(event) {
	if (event == null) event = window.event;
	savedTarget = null;
	dragOK = false;
	return false;
}


/* Helper methods that are not related to the widgetry */

// Return an array of parameter values for the given parameter.
// If the parameter is in a group, respect any preceding parameter
// selections.
// If the REST service call to get the values has not returned,
// return null rather than an empty array.
function getParameterValues(parameter) {
	var group = findGroup(parameter.name);
	if (group == null) {
		var index = getParameterIndex(parameter.name);
		if (parameterPotentialValues[index] == null || parameterPotentialValues[index] == undefined) {
			return null;
		}
		return parameterPotentialValues[index];
	} else {
		var index = getParameterIndex(group.name);
		var current = parameterPotentialValues[index];
		if (current == null || current == undefined) {
			return null;
		}
		return computeNewLabelsOrValues(parameter.name, group, current, false);
	}
}
function getParameterLabels(parameter) {
	var group = findGroup(parameter.name);
	if (group == null) {
		var index = getParameterIndex(parameter.name);
		if (parameterPotentialValues[index] == null || parameterPotentialValues[index] == undefined) {
			return null;
		}
		return parameterPotentialLabels[index];
	} else {
		var index = getParameterIndex(group.name);
		var current = parameterPotentialValues[index];
		if (current == null || current == undefined) {
			return null;
		}
		return computeNewLabelsOrValues(parameter.name, group, current, true);
	}
}

// Compute and return a list of values or labels to populate the combo for the named
// parameter. This list is based on the selected values from prior parameters
// in the group. For the purposes of computation, a prior parameter which has
// no selected values is treated like a wildcard for subsequent parameters (unless
// there is a default value specified, in which case the default value is used).
// The list is not sorted and may contain duplicates.
function computeNewLabelsOrValues(parameterName, group, parameterStruct, useLabels) {
	if (!parameterStruct || parameterStruct.values == null) return [];
	if (parameterStruct.parameterName == parameterName) {
		// We're done searching in this branch. Return the values.
		return (useLabels == true) ? parameterStruct.labels : parameterStruct.values;
	}
	// We're not there yet. Find the right child[ren] to recurse into.
	// Merge all the results into the result array.
	var result = [];
	var resultIndex = 0;
	
	// Get the currently-selected value(s) from allParameterValues array
	var allIndex = getAllParameterIndex(parameterStruct.parameterName);
	var currentlySelectedValues = allParameterValues[allIndex];
	// In the case of no selection, see if there is a default value for the report
	if (!currentlySelectedValues || currentlySelectedValues == null) {
		currentlySelectedValues = allParameterDefaultValue[allIndex];
	}
	// For each value in the struct, if that value is in the currentlySelectedValues list,
	// then recurse on the child at the same index.
	for (var i = 0; i < parameterStruct.values.length; i++) {
		if (!currentlySelectedValues || currentlySelectedValues == null || currentlySelectedValues == "" ||
			currentlySelectedValues == parameterStruct.values[i] || contains(currentlySelectedValues, parameterStruct.values[i])) {
			// If there are no values, then children or children[i] may be null
			if (parameterStruct.children == null || parameterStruct.children[i] == null) {
				continue;
			}
			var moreResults = computeNewLabelsOrValues(parameterName, group, parameterStruct.children[i], useLabels);
			for (var j = 0; j < moreResults.length; j++) {
				result[resultIndex++] = moreResults[j];
			}
		}
	}
	return result;
}

// Helper function to find the group for a given parameter name
function findGroup(parameterName) {
	for (var i = 0; i < parameterList.length; i++) {
		if (!parameterList[i].isGroup) continue;
		var children = parameterList[i].children;
		for (var j = 0; j < children.length; j++) {
			if (children[j].name == parameterName) return parameterList[i];
		}
	}
	return null;
}

// Helper function to find the next parameter in the given group, 
// after the given parameter.
function findNextParameter(group, lastParameterName) {
	if (group == null) return null;
	for (var i = 0; i < group.children.length; i++) {
		if (group.children[i].name == lastParameterName) {
			if (i < group.children.length - 1) 
				return group.children[i + 1];
			else
				return null;
		}
	}
	return null;
}

// Helper function removes duplicate values from the given list
function removeDuplicates(list) {
    var result = [];
    var resultIndex = 0;
    for (var i = 0; i < list.length; i++) {
        if (!contains(result, list[i])) result[resultIndex++] = list[i];
    }
    return result;
}

// Helper function removes duplicate values from the given list,
// and removes the corresponding values from the labels list.
function removeDuplicatesWithLabels(values, labels) {
	var labelsResult = [];
	var valuesResult = [];
	var resultIndex = 0;
	for (var i = 0; i < values.length; i++) {
		if (!contains(valuesResult, values[i])) {
			valuesResult[resultIndex] = values[i];
			labelsResult[resultIndex] = labels[i];
			resultIndex++;
		}
	}
	return {
		values: valuesResult,
		labels: labelsResult
	};
}


// Helper function returns whether the given list contains the given item
function contains(list, item) {
    for (var i = 0; i < list.length; i++) {
        if (list[i] == item) {
        	return true;
        }
    }
    return false;
}

// Return whether or not all required parameters have values, and
// are of the correct type.
function hasAllRequiredParameters() {
	for (var i = 0; i < allParameterValues.length; i++) {
		// If there is no value but one is required, return false.
		if (allParameterAllowNull[i] == false) {
			if (allParametersIsValueSet[i] == true) {
				if (!allParameterValues[i] || allParameterValues[i] == '') {
					return false;
				}
			} else {
				if ((!allParameterValues[i] || allParameterValues[i] == '') && 
			    	(!allParameterDefaultValue[i] || allParameterDefaultValue[i] == '')) {
					return false;
				}
			}
		}
		// If there is a value but it is of the wrong type, return false.
		var val = allParameterValues[i];
		if (val == null) val = allParameterDefaultValue[i];
		if (val) {
			// Currently we only validate integer fields
			if (allParameterList[i].type == TYPE_INTEGER) {
			    var value = val;
			    if (value != "") {
				    if (value[0] == "'") {
				    	value = value.substr(1, value.length - 2);
				    }
				    var valueInt = parseInt(value);
				    if (isNaN(valueInt)) {
				    	return false;
				    }
				}
			}
		}
	}
    return true;
}

// Helper method to get the currently-selected value for a given parameter.
function getCurrentValues(parameter) {
	var index = getAllParameterIndex(parameter.name);
	if (index == -1) return null;
	return allParameterValues[index];
}

// Helper method to get the default value specified in the report design
// for the given parameter.
function getDefaultValues(parameter) {
	var index = getAllParameterIndex(parameter.name);
	if (index == -1) return null;
	return allParameterDefaultValue[index];
}

// Helper method to get the currently-selected labels for a given parameter.
// The elements of the array may or may not be the same as getCurrentValues(),
// depending on whether labels were specified and are available.
// If there are no values saved for the given parameter, use the default values
// specified in the report. If there are no default values, return an empty array.
function getCurrentOrDefaultLabels(parameter) {
	var currentValues = getCurrentValues(parameter);
	var defaultValues = getDefaultValues(parameter);
	var index = getAllParameterIndex(parameter.name);
	if (index == -1) return currentValues;

	if (currentValues == null && defaultValues == null) return null;		
	var potentialLabels = getParameterLabels(parameter);
	var potentialValues = getParameterValues(parameter);
	if (potentialLabels == null || potentialValues == null) {
		return currentValues;
	}
	
	if (currentValues == null) {
		currentValues = defaultValues;
	}
	if (!(currentValues instanceof Array)) {
		// Only one value - look for the label
		for (var i = 0; i < potentialValues.length; i++) {
			if (potentialValues[i] == currentValues) {
				return potentialLabels[i];
			}
		}
		return currentValues;
	}
	var labels = [];
	for (var i = 0; i < currentValues.length; i++) {
		for (var j = 0; j < potentialValues.length; j++) {
			if (potentialValues[j] == currentValues[i]) {
				labels[i] = potentialLabels[j];
			}
		}
		if (labels[i] == null) {
			labels[i] = currentValues[i];
		}
	}
	return labels;
}

// Helper method to return the index of the named parameter or parameter group
// in the parameterList array.
function getParameterIndex(parameterName) {
	for (var i = 0; i < parameterList.length; i++) {
		if (parameterList[i].name == parameterName) {
			return i;
		}
	}
	return -1;
}

// Helper method to return the index of the named parameter in the
// flat list allParametersList.
function getAllParameterIndex(parameterName) {
	for (var i = 0; i < allParameterList.length; i++) {
		if (allParameterList[i].name == parameterName) {
			return i;
		}
	}
	return -1;
}

// Sort the pairs of lists. Sort alphabetically according to
// the labels list and keep the values list in the corresponding order.
function sort(labels, values) {
	var toSort = [];
	for (var i = 0; i < values.length; i++) {
		toSort[i] = { label: labels[i], value: values[i] };
	}
	function sortFunction(a, b) {
		return a.label < b.label ? -1 : (a.label > b.label ? 1 : 0);
	}
	var sorted = toSort.sort(sortFunction);
	var sortedLabels = [];
	var sortedValues = [];
	for (var i = 0; i < sorted.length; i++) {
		sortedLabels[i] = sorted[i].label;
		sortedValues[i] = sorted[i].value;
	}
	return {
		labels: sortedLabels,
		values: sortedValues
	};
}

// Sort the pairs of lists. Each list is comprised of hierarchical,
// slash-separated values.
function sortTree(labels, values) {
	var toSort = [];
	for (var i = 0; i < values.length; i++) {
		toSort[i] = { label: labels[i], value: values[i], tokenized: tokenize(labels[i] != null ? labels[i] : values[i]) };
	}
	function sortFunction(a, b) {
		return compare(a.tokenized, b.tokenized, 0);
	}
	var sorted = toSort.sort(sortFunction);
	var sortedLabels = [];
	var sortedValues = [];
	for (var i = 0; i < sorted.length; i++) {
		sortedLabels[i] = sorted[i].label;
		sortedValues[i] = sorted[i].value;
	}
	return {
		labels: sortedLabels,
		values: sortedValues
	};
}

function compare(tokenArray1, tokenArray2, index) {
	if (index >= tokenArray1.length || index >= tokenArray2.length) {
		if (index >= tokenArray1.length && index >= tokenArray2.length) return 0;
		return (index >= tokenArray1.length) ? -1 : 1;
	}
	if (tokenArray1[index] == tokenArray2[index]) {
		return compare(tokenArray1, tokenArray2, index + 1);
	}
	return (tokenArray1[index] < tokenArray2[index]) ? -1 : 1;
}

// Tokenize the hierarchical, slash-separated string. Return
// an array of tokens. If the string isn't slash-separated, 
// return an array with the string. There may not be a leading slash.
function tokenize(string) {
	var result = [];
	var index = string.indexOf("/");
	if (index == -1) {
		return [string];
	}
	if (index != 0) {
		string = "/" + string;
		index = 0;
	}
	var nextIndex;
	var i = 0;
	while (true) {
		nextIndex = string.indexOf("/", index + 1);
		if (nextIndex == -1) {
			// Last segment
			result[i++] = string.substr(index + 1);
			return result;
		} else {
			result[i++] = string.substring(index + 1, nextIndex);
		}
		index = nextIndex;
	}
	return null;
}

function getLabelFor(values) {
	if (values == null || values == "''" || values == "") return "(No selection)";
	if (!(values instanceof Array)) {
		values = [values];
	}
	if (values.length == 0) {
		return "(No selection)";
	} else {
		var label = "";
		for (var i = 0; i < values.length; i++) {
			if (label != "") {
				label += ", ";
			}
			if (values[i] == MY_TEAM_AREAS) {
				label += "My Team Areas";
			} else if (values[i] == CURRENT_ITERATION) {
				label += "Current Iteration";
			} else if (values[i] == CURRENT_CONTRIBUTOR) {
				label += "Current Contributor";
			} else if (values[i] instanceof Date) {
				label += dojo.date.locale.format(values[i], {selector: "date"});
			} else {
				label += values[i];
			}
		}
		if (label.length > 40) {
			label = "(" + values.length + " items selected)";
		}
		return label;
	}
}

function isIe() {
	return navigator.userAgent != null && navigator.userAgent.indexOf("MSIE") != -1;
}

function isIe7() {
	return isIe() && (ie7 == true);
}

function isSpecialParameter(parameterName) {
    if (parameterName.indexOf("ProjectAreaName") == 0) return true;
    if (parameterName.indexOf("ProjectAreaItemId") == 0) return true;
    return false;
}

// Make a copy of the allParameterValues so we can fill in undefined and null values.
// Also, flatten out into a single string for each one, including commas where necessary.
// Dates will be converted into strings appropriate to send to BIRT.
// Return a structure with parameterNames and parameterValues as fields.
function prepareParameters(oldParameterValues) {
    var newAllParameterValues = [];
    var newAllParameterNames = [];

    var newIndex = 0;
    for (var i = 0; i < oldParameterValues.length; i++) {
    	if (!oldParameterValues[i] || oldParameterValues[i] == null || ((oldParameterValues[i] instanceof Array) && (oldParameterValues.length == 0))) {
    		// Do nothing
    	} else if (oldParameterValues[i] instanceof Array) {
    		newAllParameterValues[newIndex] = '';
    		for (var j = 0; j < oldParameterValues[i].length; j++) {
    			if (j > 0) newAllParameterValues[newIndex] += ",";
    			newAllParameterValues[newIndex] += "'" + doubleQuote(oldParameterValues[i][j]) + "'";
    		}
    		newAllParameterNames[newIndex] = allParameterList[i].name;
    		newIndex++;
    	} else if (oldParameterValues[i] instanceof Date) {
    		newAllParameterValues[newIndex] = getBirtDate(oldParameterValues[i]);
    		newAllParameterNames[newIndex] = allParameterList[i].name;
    		newIndex++;
    	} else {
    		newAllParameterValues[newIndex] = "'" + doubleQuote(oldParameterValues[i]) + "'";
    		newAllParameterNames[newIndex] = allParameterList[i].name;
    		newIndex++;
    	}
    }
    return {
    	names: newAllParameterNames,
    	values: newAllParameterValues
    };
}

function getBirtDate(date) {
	return "" + date.getDate() + "/" + (date.getMonth() + 1) + "/" + date.getFullYear();
}

// Given the parameter group DTO, walk through it any extract any flags fields
function computeFlagsForGroup(parameterGroup) {
	if (parameterGroup.parameters != null) {
		for (var i = 0; i < parameterGroup.parameters.length; i++) {
			computeFlagsForParameter(parameterGroup.parameters[i]);
		}
	}
}
function computeFlagsForParameter(parameter) {
	var index = getAllParameterIndex(parameter.parameterName);
	allParameterFlags[index] = parameter.flags;
	if (parameter.children != null) {
		for (var i = 0; i < parameter.children.length; i++) {
			computeFlagsForParameter(parameter.children[i]);
		}
	}
}

function setDirty(dirty) {
	// TODO: Also toggle the availability of the Set As default toolbar
	if (dirty && !isDirty) {
		// Newly dirty, add a dirty label
		var div = document.getElementById("dirtyMarker");
		div.innerHTML = "*";
	} else if (!dirty && isDirty) {
		// Newly clean, remove the dirty label
		var div = document.getElementById("dirtyMarker");
		div.innerHTML = "";
	}
	isDirty = dirty;
}

// In IE, selects always have borders. Because we give them fixed heights,
// the height of the checkbox lists should be modified to match.
function fixListHeight(id) {
	if (isIe()) {
		var element = document.getElementById(id);
		if (element == null) return;
		element.style.height = 154;
	}
}

function fixComboWidth() {
	if (isIe7()) {
		var nodes = document.getElementsByTagName("select");
		for (var i = 0; i < nodes.length; i++) {
			var select = nodes[i];
			if (select.className == "singleSelectCombo") {
				select.className = "singleSelectComboIe7";
			}
		}
	}
}

function equals(one, two) {
	if (one == two) return true;
	if (one == undefined || two == undefined) return false;
	if (one == null || two == null) return false;
	if ((one instanceof Array) && (two instanceof Array)) {
		if (one.length != two.length) return false;
		for (var i = 0; i < one.length; i++) {
			if (one[i] != two[i]) return false;
		}
		return true;
	}
	return false;
}

/**
 * Given a slash-delimited string with a leading slash, return
 * which level the string represents. "/Jazz" returns 1.
 * "/Jazz/Reports" returns 2.
 * If a string doesn't have any leading slash, return 1.
 */
function treeLevel(string) {
	if (string.indexOf("/") != 0) {
		string = "/" + string;
	}
	var result = 0;
	var index = string.indexOf("/");
	while (index != -1) {
		result++;
		string = string.substr(index + 1);
		index = string.indexOf("/");
	}
	return result == 0 ? 1 : result;
}

/**
 * Given a slash-delimited string with a leading slash, return
 * the substring to use as a label. "/Jazz" returns "Jazz".
 * "/Jazz/Reports" returns "Reports".
 * If the string doesn't have any leading slash, return the string.
 */
function treeString(string) {
	var lastIndex = string.lastIndexOf("/");
	if (lastIndex == -1) return string;
	return string.substr(lastIndex + 1);
}

/**
 * Given a slash-delimited string with a leading slash,
 * return an array of tokens representing each segment of
 * the string.
 * If the string doesn't have any leading slash, return an
 * array of size one with the string. 
 */
function treeTokens(string) {
	var result = [];
	var firstIndex = string.indexOf("/");
	if (firstIndex == -1) {
		result[0] = string;
		return result;
	}
	while (firstIndex != -1) {
		var nextIndex = string.indexOf("/", firstIndex + 1);
		if (nextIndex == -1) {
			// We're on the last token
			var token = string.substring(firstIndex + 1);
			result[result.length] = token;
			return result;
		}
		// There are more tokens.
		var token = string.substring(firstIndex + 1, nextIndex);
		result[result.length] = token;
		firstIndex = nextIndex;
	}
	// Should never get here
	return result;
}

function stringToHTML(string) {
	return string.replace(/[&<>]/g,
      function(m) {
		var map = {
		  "&": "amp",
		  "<": "lt",
		  ">": "gt"
		};
		return "&" + map[m] + ";";
	});
 } 
 
/**
 * Given a sorted, hierarchical list of values and labels,
 * return a data structure representing the root node of a
 * tree. 
 * 
 * {
 *     value: (value string),
 *     label: (label string),
 *     level: (integer),
 *     fabricated: [true|FALSE],
 *     children: (array of structures similar to this)
 * }
 * 
 * The root node has no value or label, only children.
 */
function createTree(values, labels) {
	var result = {
		level: 0,
		fabricated: true,
		children: []
	};
	for (var i = 0; i < values.length; i++) {
		insertTreeNode(values[i], labels[i], result);
	}
	return result;
}

/**
 * Create and insert a node into the given tree to represent
 * the given value/label pair. Note that this may insert more
 * than one node in the case that intermediate nodes need to
 * be fabricated.
 * 
 * This function assumes that all callers have sorted the values
 * and labels beforehand, and therefore there is no chance that
 * a fabricated node will turn into a real node later. It also
 * means that each node can always be inserted at the end of a
 * collection.
 */
function insertTreeNode(value, label, tree) {
	// Find out where to insert it
	var parent = findOrCreateParent(label, tree);
	// Construct the node
	var newNode = {
		value: value,
		label: treeString(label),
		level: treeLevel(label),
		children: []
	};
	// Insert it
	parent.children[parent.children.length] = newNode;
}

function findOrCreateParent(label, tree) {
	var tokens = treeTokens(label); 
	var result = tree;
	for (var i = 0; i < tokens.length - 1; i++) {
		// For each segment, look for a match in
		// result. If it doesn't exist, create it
		// and insert it.
		var childMatch = null;
		for (var j = 0; j < result.children.length; j++) {
			if (result.children[j].label == tokens[i]) {
				childMatch = result.children[j];
				break;
			}
		}
		if (childMatch == null) {
			// We didn't find it, create and insert it now.
			// It doesn't need a value.
			var newNode = {
				label: tokens[i],
				level: i+1,
				fabricated: true,
				children: []
			}
			result.children[result.children.length] = newNode;
			childMatch = newNode;
		}
		// Now that we found (or created) the next child in
		// the path, move on to the next one.
		result = childMatch;
	}
	return result;
}

function visit(treeNode, callbackFunction, id, reservedId) {
	if (id == reservedId) {
		id++;
	}
	if (treeNode.level > 0) {
		callbackFunction(treeNode.label, treeNode.value, treeNode.level, (treeNode.fabricated == true) ? true : false, id);
		id++;
	}
	if (treeNode.children != null) {
		for (var i = 0; i < treeNode.children.length; i++) {
			id = visit(treeNode.children[i], callbackFunction, id, reservedId);
		}
	}
	return id;
}

function escapeQuotes(string) {
	if (typeof(string) != "string") {
		return string;
	}
	return string
		.replace(/\'/g, "&#39;")
		.replace(/\"/g, "&quot;"); 
}

function doubleQuote(string) {
	if (typeof(string) != "string") {
		return string;
	}
	return string.replace(/\'/g, "\'\'");
}