/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.process.web.ui.internal.ProjectAreaListPage");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("dojo.cookie");

dojo.require("net.jazz.ajax.ui.PlatformUI");

dojo.require("com.ibm.team.process.web.client.internal.ProcessClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.init.InitializationData");

var ProcessClient = com.ibm.team.process.web.client.internal.ProcessClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var InitializationData = com.ibm.team.repository.web.client.init.InitializationData;
var getAuthenticatedContributor = com.ibm.team.repository.web.client.session.getAuthenticatedContributor;

var MANAGE_PROJECT_AREAS_ACTION_URI = "#action=jazz.viewPage&id=com.ibm.team.process.ProjectAreaManagement";
var SHOWARCHIVED_COOKIE = "JazzProjectsShowArchived";

dojo.declare(
	"com.ibm.team.process.web.ui.internal.ProjectAreaListPage",
	[dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.process.web", "ui/internal/templates/ProjectAreaListPage.html"),

	constructor: function() {
		this._manageHref = net.jazz.ajax._contextRoot + "/admin" + MANAGE_PROJECT_AREAS_ACTION_URI;
	},

	postCreate: function() {
		var ActionRegistry = net.jazz.ajax.ui.PlatformUI.getWorkbench().getActionRegistry();
		ActionRegistry.registerAction("com.ibm.team.process.web._listProjects", this, "_runDefaultAction");
	},
	
	_runDefaultAction: function() {
		dojox.data.dom.removeChildren(this.myProjects);
		dojox.data.dom.removeChildren(this.otherProjects);
		dojox.data.dom.removeChildren(this.archivedProjects);
		
		var authContrib = getAuthenticatedContributor();
		var authContribId;
		if (authContrib != null) {
			authContribId = authContrib.userId;
		}
		var srh = new ServiceResponseHandler(this, "populateProjectAreas", "handleError");
		ProcessClient.getAllProjectAreas(srh, {userId: authContribId});
		var permissionInitializer = InitializationData.get("com.ibm.team.repository.service.canGrantPermissionInitializer");
		if (permissionInitializer != null && permissionInitializer.canGrantPermission) {
			dojo.removeClass(this.adminLinkArea,"hidden");
			dojo.removeClass(this.manageLink,"hidden");
			dojo.addClass(this.adminContact,"hidden");
		} else {
			dojo.addClass(this.adminLinkArea,"hidden");
			dojo.addClass(this.manageLink,"hidden");
			dojo.removeClass(this.adminContact,"hidden");
		}
	},
	
	populateProjectAreas: function (projectAreas) {
		if (projectAreas) {
			projectAreas = projectAreas.sort(function (a,b) { return a.name > b.name ? 1 : a.name < b.name ? -1 : 0});
			var myProjects = new Array();
			var archivedProjects = new Array();
			for (var i = 0; i < projectAreas.length; i++) {
				if (projectAreas[i].archived) {
					archivedProjects.push(projectAreas[i]);
					projectAreas.splice(i,1);
					i--;
				} else if (projectAreas[i].myProjectArea) {
					myProjects.push(projectAreas[i]);
					projectAreas.splice(i,1);
					i--;
				}
			}
		}
		if (!projectAreas || (projectAreas.length == 0 && archivedProjects.length == 0 && myProjects.length == 0)) {
			dojo.removeClass(this.noProjectAreasMessage, "hidden");
		} else {
			if (myProjects.length > 0){
				this.populateArea(this.myProjects, myProjects)
			} else {
				dojo.addClass(this.myProjects.parentNode,"hidden");
			}
			
			if (projectAreas && projectAreas.length > 0){
				this.populateArea(this.otherProjects, projectAreas)
			} else {
				dojo.addClass(this.otherProjects.parentNode,"hidden");
			}
			
			if (archivedProjects.length > 0){
				dojo.removeClass(this.filterArea,"hidden");			
				var storedPref = dojo.cookie(SHOWARCHIVED_COOKIE);
				if (storedPref) {
					if ((!projectAreas || projectAreas.length == 0) && myProjects.length == 0) dojo.addClass(this.noProjectAreasMessage, "hidden");
					this.showArchivedCheckbox.checked = "checked";
					this.populateArea(this.archivedProjects, archivedProjects, false);
				} else {
					if ((!projectAreas || projectAreas.length == 0) && myProjects.length == 0) dojo.removeClass(this.noProjectAreasMessage, "hidden");
					this.showArchivedCheckbox.checked = null;
					this.populateArea(this.archivedProjects, archivedProjects, true);
				}
			}	
		}
	},
	_onCheckboxChange: function(event) {
		if (event.target.checked) {
			dojo.cookie(SHOWARCHIVED_COOKIE, "checked", { expires: 365 });
			dojo.removeClass(this.archivedProjectsSection,"hidden");
			if (dojo.hasClass(this.otherProjects.parentNode, "hidden") && dojo.hasClass(this.myProjects.parentNode,"hidden"))
				dojo.addClass(this.noProjectAreasMessage, "hidden");
		} else {
			dojo.cookie(SHOWARCHIVED_COOKIE, "");
			dojo.addClass(this.archivedProjectsSection,"hidden");
			if (dojo.hasClass(this.otherProjects.parentNode, "hidden") && dojo.hasClass(this.myProjects.parentNode,"hidden"))
				dojo.removeClass(this.noProjectAreasMessage, "hidden");
		}
	},
	populateArea: function(area, data, hide){
		if (area && area.parentNode) {
			if (!hide && area.parentNode)
				dojo.removeClass(area.parentNode,"hidden");
			else
				dojo.addClass(area.parentNode,"hidden");
		}
		var curUrl = new dojo._Url(location.href);
		
		if (curUrl.query) {  //check for debug mode
			var urlSuffix = "?" + curUrl.query;	
		} else {
			var urlSuffix = "";
		} 
				
		for (var i = 0; i < data.length; i++)
			area.appendChild(this.createProjectAreaRow(data[i].name, data[i].summary, data[i].description, urlSuffix));
	},
	handleError: function (err) {
		dojox.data.dom.removeChildren(this.projectAreaList);
		this.displayErrorMessage("Error fetching project information: " + err);
	},
	displayErrorMessage: function (msg) {
		this.displayMessage(msg, "errorMsg");
	},
	displayMessage: function (msg, cssClass) {
		dojox.data.dom.removeChildren(this.messageBox);
		this.messageBox.className = cssClass;
		this.messageBox.appendChild(document.createTextNode(msg));
	},
	createProjectAreaRow: function (projectAreaName, projectAreaSummary, projectAreaDetails, urlSuffix) {
		var row = document.createElement("div");
		row.className = "project";
		
		// create project area link
		var linkArea = document.createElement("h3");
		var link = document.createElement("a");
		link.appendChild(document.createTextNode(projectAreaName));
		linkArea.appendChild(link);
		var encodedName = encodeURI(projectAreaName);
		// encodeURI doesn't encode "?".  We need to force the encoding here.
		encodedName = encodedName.replace(/\?/g, "%3F");
		link.href = net.jazz.ajax._contextRoot + "/web/projects/" + encodedName + urlSuffix;
		row.appendChild(linkArea);
		
		if (projectAreaSummary) {
			row.appendChild(document.createTextNode(projectAreaSummary));
		}
		if (projectAreaSummary && projectAreaDetails) {
			row.appendChild(document.createElement("br"));
			row.appendChild(document.createElement("br"));
		}
		if (projectAreaDetails) {
			var lines = projectAreaDetails.split(/\n/);
			for (var i=0; i < lines.length; i++) {
				if (i > 0) {
					row.appendChild(document.createElement("br"));
				}
				row.appendChild(document.createTextNode(lines[i]));
			}
		}
		
		return row;
	}
});

})();